@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright 2015 Randal S. Olson
+Copyright 2016 Randal S. Olson
 
 This file is part of the TPOT library.
 
@@ -14,7 +14,7 @@ The TPOT library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 You should have received a copy of the GNU General Public License along with
-the Twitter Bot library. If not, see http://www.gnu.org/licenses/.
+the TPOT library. If not, see http://www.gnu.org/licenses/.
 """
 
 # Utility functions that convert the current optimized pipeline into its corresponding Python code
@@ -22,7 +22,6 @@ the Twitter Bot library. If not, see http://www.gnu.org/licenses/.
 
 import deap
 
-
 def replace_mathematical_operators(exported_pipeline):
     """Replace all of the mathematical operators with their results for use in export(self, output_file_name)
 
@@ -37,7 +36,6 @@ def replace_mathematical_operators(exported_pipeline):
        The current optimized pipeline after replacing the mathematical operators
 
     """
-
     while True:
         for i in range(len(exported_pipeline) - 1, -1, -1):
             node = exported_pipeline[i]
@@ -64,8 +62,8 @@ def replace_mathematical_operators(exported_pipeline):
 
     return exported_pipeline
 
-def unroll_nested_fuctions_calls(exported_pipeline):
-    """Unroll the nested function calls into serial code for use in export(self, output_file_name)
+def unroll_nested_fuction_calls(exported_pipeline):
+    """Unroll the nested function calls into serial code for use in TPOT.export()
 
     Parameters
     ----------
@@ -80,7 +78,6 @@ def unroll_nested_fuctions_calls(exported_pipeline):
        List of operators in the current optimized pipeline
 
     """
-
     pipeline_list = []
     result_num = 1
     while True:
@@ -103,8 +100,59 @@ def unroll_nested_fuctions_calls(exported_pipeline):
             break
     return exported_pipeline, pipeline_list
 
+def generate_import_code(pipeline_list):
+    """Generate all library import calls for use in TPOT.export()
+
+    Parameters
+    ----------
+    pipeline_list:
+       List of operators in the current optimized pipeline
+
+    Returns
+    -------
+    pipeline_text:
+       The Python code that imports all required library used in the current optimized pipeline
+
+    """
+    # operator[1] is the name of the operator
+    operators_used = set([operator[1] for operator in pipeline_list])
+
+    pipeline_text = '''import numpy as np
+import pandas as pd
+
+from sklearn.cross_validation import StratifiedShuffleSplit
+'''
+
+    if '_variance_threshold' in operators_used: pipeline_text += 'from sklearn.feature_selection import VarianceThreshold\n'
+    if '_select_kbest' in operators_used: pipeline_text += 'from sklearn.feature_selection import SelectKBest\n'
+    if '_select_fwe' in operators_used: pipeline_text += 'from sklearn.feature_selection import SelectFwe\n'
+    if '_select_percentile' in operators_used: pipeline_text += 'from sklearn.feature_selection import SelectPercentile\n'
+    if ('_select_percentile' in operators_used or
+        '_select_kbest' in operators_used or
+        '_select_fwe' in operators_used): pipeline_text += 'from sklearn.feature_selection import f_classif\n'
+    if '_rfe' in operators_used: pipeline_text += 'from sklearn.feature_selection import RFE\n'
+    if '_standard_scaler' in operators_used: pipeline_text += 'from sklearn.preprocessing import StandardScaler\n'
+    if '_robust_scaler' in operators_used: pipeline_text += 'from sklearn.preprocessing import RobustScaler\n'
+    if '_polynomial_features' in operators_used: pipeline_text += 'from sklearn.preprocessing import PolynomialFeatures\n'
+    if '_pca' in operators_used: pipeline_text += 'from sklearn.decomposition import RandomizedPCA\n'
+    if '_decision_tree' in operators_used: pipeline_text += 'from sklearn.tree import DecisionTreeClassifier\n'
+    if '_random_forest' in operators_used: pipeline_text += 'from sklearn.ensemble import RandomForestClassifier\n'
+    if '_logistic_regression' in operators_used: pipeline_text += 'from sklearn.linear_model import LogisticRegression\n'
+    if '_svc' in operators_used or '_rfe' in operators_used: pipeline_text += 'from sklearn.svm import SVC\n'
+    if '_knnc' in operators_used: pipeline_text += 'from sklearn.neighbors import KNeighborsClassifier\n'
+    if '_xgradient_boosting' in operators_used: pipeline_text += 'from xgboost import XGBClassifier\n'
+
+    pipeline_text += '''
+# NOTE: Make sure that the class is labeled 'class' in the data file
+tpot_data = pd.read_csv('PATH/TO/DATA/FILE', sep='COLUMN_SEPARATOR')
+training_indices, testing_indices = next(iter(StratifiedShuffleSplit(tpot_data['class'].values, n_iter=1, train_size=0.75, test_size=0.25)))
+
+'''
+
+    return pipeline_text
+
 def replace_function_calls(pipeline_list):
-    """Replace the function calls with their corresponding Python code for use in export(self, output_file_name)
+    """Replace the function calls with their corresponding Python code for use in TPOT.export()
 
     Parameters
     ----------
@@ -114,7 +162,7 @@ def replace_function_calls(pipeline_list):
     Returns
     -------
     operator_text:
-       The python code corresponding to the function calls in the current optimized pipeline
+       The Python code corresponding to the function calls in the current optimized pipeline
 
     """
     operator_text = ''
@@ -122,7 +170,7 @@ def replace_function_calls(pipeline_list):
         operator_num = int(operator[0].strip('result'))
         result_name = operator[0]
         operator_name = operator[1]
-        
+
 
         # Make copies of the data set for each reference to ARG0
         if operator[2] == 'ARG0':
@@ -134,7 +182,7 @@ def replace_function_calls(pipeline_list):
             operator_text += '\n{} = tpot_data.copy()\n'.format(operator[3])
 
         # Replace the TPOT functions with their corresponding Python code
-        if operator_name == 'decision_tree':
+        if operator_name == '_decision_tree':
             max_features = int(operator[3])
             max_depth = int(operator[4])
 
@@ -155,7 +203,7 @@ def replace_function_calls(pipeline_list):
                 operator_text += '{} = {}\n'.format(result_name, operator[2])
             operator_text += '''{0}['dtc{1}-classification'] = dtc{1}.predict({0}.drop('class', axis=1).values)\n'''.format(result_name, operator_num)
 
-        elif operator_name == 'random_forest':
+        elif operator_name == '_random_forest':
             num_trees = int(operator[3])
             max_features = int(operator[4])
 
@@ -178,7 +226,7 @@ def replace_function_calls(pipeline_list):
                 operator_text += '{} = {}\n'.format(result_name, operator[2])
             operator_text += '''{0}['rfc{1}-classification'] = rfc{1}.predict({0}.drop('class', axis=1).values)\n'''.format(result_name, operator_num)
 
-        elif operator_name == 'logistic_regression':
+        elif operator_name == '_logistic_regression':
             C = float(operator[3])
             if C <= 0.:
                 C = 0.0001
@@ -190,7 +238,7 @@ def replace_function_calls(pipeline_list):
                 operator_text += '{} = {}\n'.format(result_name, operator[2])
             operator_text += '''{0}['lrc{1}-classification'] = lrc{1}.predict({0}.drop('class', axis=1).values)\n'''.format(result_name, operator_num)
 
-        elif operator_name == 'svc':
+        elif operator_name == '_svc':
             C = float(operator[3])
             if C <= 0.:
                 C = 0.0001
@@ -202,7 +250,7 @@ def replace_function_calls(pipeline_list):
                 operator_text += '{} = {}\n'.format(result_name, operator[2])
             operator_text += '''{0}['svc{1}-classification'] = svc{1}.predict({0}.drop('class', axis=1).values)\n'''.format(result_name, operator_num)
 
-        elif operator_name == 'knnc':
+        elif operator_name == '_knnc':
             n_neighbors = int(operator[3])
             if n_neighbors < 2:
                 n_neighbors = 2
@@ -216,7 +264,7 @@ def replace_function_calls(pipeline_list):
                 operator_text += '{} = {}\n'.format(result_name, operator[2])
             operator_text += '''{0}['knnc{1}-classification'] = knnc{1}.predict({0}.drop('class', axis=1).values)\n'''.format(result_name, operator_num)
 
-        elif operator_name == 'gradient_boosting':
+        elif operator_name == '_xgradient_boosting':
             learning_rate = float(operator[3])
             n_estimators = int(operator[4])
             max_depth = int(operator[5])
@@ -232,12 +280,12 @@ def replace_function_calls(pipeline_list):
             if max_depth < 1:
                 max_depth = None
 
-            operator_text += '\n# Perform classification with a gradient boosting classifier'
-            operator_text += '\ngbc{} = GradientBoostingClassifier(learning_rate={}, n_estimators={}, max_depth={})\n'.format(operator_num, learning_rate, n_estimators, max_depth)
-            operator_text += '''gbc{0}.fit({1}.loc[training_indices].drop('class', axis=1).values, {1}.loc[training_indices, 'class'].values)\n'''.format(operator_num, operator[2])
+            operator_text += '\n# Perform classification with an eXtreme gradient boosting classifier'
+            operator_text += '\nxgbc{} = XGBClassifier(learning_rate={}, n_estimators={}, max_depth={})\n'.format(operator_num, learning_rate, n_estimators, max_depth)
+            operator_text += '''xgbc{0}.fit({1}.loc[training_indices].drop('class', axis=1).values, {1}.loc[training_indices, 'class'].values)\n'''.format(operator_num, operator[2])
             if result_name != operator[2]:
                 operator_text += '{} = {}\n'.format(result_name, operator[2])
-            operator_text += '''{0}['gbc{1}-classification'] = gbc{1}.predict({0}.drop('class', axis=1).values)\n'''.format(result_name, operator_num)
+            operator_text += '''{0}['xgbc{1}-classification'] = xgbc{1}.predict({0}.drop('class', axis=1).values)\n'''.format(result_name, operator_num)
 
         elif operator_name == '_combine_dfs':
             operator_text += '\n# Combine two DataFrames'
